"""
Author:
     Russell Parkinson at Open Spatial Solutions for Geological Survey
     of Canada (GSC)

     Revised for publication by Charles Logan, Geological Survey
     of Canada (GSC)

Contact Information:
     William Parkinson
     Open Spatial Solutions
     89 Forward Ave. Suit 1211
     613 454 8332
     will.parkinson@openspatialsolutions.com
     www.openspatialsolutions.com

Dependencies:
     PIL (This Package must be installed, PIL-1.1.7.win32-py2.7.exe)
     re
     pyexiv2 (This Package must be installed, pyexiv2-0.3.2-setup.exe)
     arcpy
     shutil
     sys
     os

Be sure to install the dependencies before attempting to use this tool. This tool is
executed via toolbox (EXIF Image viewer.tbx) for ArcGIS 10.1.  Only ArcView Licence level is
required for use.

***Supported for ARC 10.0 and ARC 10.1 ONLY***
"""

from PIL import Image
import arcpy, os, sys, shutil, re


#This is the location of the Tool Package(Toolbox, images, and script)
whereAmI=os.path.split(os.path.realpath(__file__))[0]
#The first if looks for the advanced version of the script

#Setting the Parameters for the script
infcSTN = arcpy.GetParameterAsText (0)
infcPHO = arcpy.GetParameterAsText (1)
useslected = arcpy.GetParameterAsText(2)
selectionSQL = arcpy.GetParameterAsText(3)
selectbylocation = arcpy.GetParameterAsText(4)
pselectionSQL = arcpy.GetParameterAsText(5)
stationName = arcpy.GetParameterAsText (6)
photoName = arcpy.GetParameterAsText (7)
inputPhotoDB = arcpy.GetParameterAsText (8)
outputPhotoKML = arcpy.GetParameterAsText (9)
fields = arcpy.GetParameterAsText(10)
saveOutFile = arcpy.GetParameterAsText (11)
kmlName = arcpy.GetParameterAsText(12)


arcpy.AddMessage(selectionSQL)
#Searches to ensure the shapefile is loaded into the arcmap browser
if(arcpy.Describe(infcSTN).dataType=="ShapeFile"):
    arcpy.AddError("Please load shapefile into ArcMap")

#Saves the file as the name of the shapefile if not specified by the user
if saveOutFile == "":
    saveOutFile = os.path.splitext(infcSTN)[0]


#creating the working directory from the parameters
if not os.path.exists(outputPhotoKML + "/KMLPhotos"):
    os.makedirs(outputPhotoKML + "/KMLPhotos")
else:
    arcpy.AddError("Please choose another folder for your output or please delete old output data folders")
    sys.exit(1)
if not os.path.exists(outputPhotoKML + "/KMLData"):
    os.makedirs(outputPhotoKML + "/KMLData")
else:
    arcpy.AddError("Please choose another folder for your output or please delete old output data folders")
    sys.exit(1)

#Copying PNG files to working directory
shutil.copy(whereAmI+"\\KMLData\\green.png", outputPhotoKML + "\\KMLData")
shutil.copy(whereAmI+"\\KMLData\\green_highlight.png", outputPhotoKML + "\\KMLData")


#Creating a New KML and begin writing to it
f=open(outputPhotoKML+"\\"+saveOutFile+".kml", 'w')
f.write('''<?xml version="1.0" encoding="UTF-8"?>
<kml xmlns="http://www.opengis.net/kml/2.2">
<Document>
  <name>%s</name>
    <Style id="highlightPlacemark">
      <IconStyle>
        <Icon>
          <href>KMLData/green_highlight.png</href>
        </Icon>
      </IconStyle>
    </Style>
    <Style id="normalPlacemark">
      <IconStyle>
        <Icon>
          <href>KMLData/green.png</href>
        </Icon>
      </IconStyle>
	  <LabelStyle>
            <scale>0</scale>
      </LabelStyle>
    </Style>
    <StyleMap id="exampleStyleMap">
      <Pair>
        <key>normal</key>
        <styleUrl>#normalPlacemark</styleUrl>
      </Pair>
      <Pair>
        <key>highlight</key>
        <styleUrl>#highlightPlacemark</styleUrl>
      </Pair>
    </StyleMap>''' % (kmlName))




if(fields!=''):
    fields=fields.split(';')

#Creating the describe object
desc = arcpy.Describe(infcSTN)
shapeName = desc.shapeFieldName

#Checking to ensure point file
if desc.shapeType!="Point":
    arcpy.AddError(desc.shapeType + " is incorrect feature type to be running this script. Feature classes of type 'Polygon' is required.")
    sys.exit(1)

#Determining the selction type to execute the script on
selectionType='NEW_SELECTION'

if useslected=="true":
    selectionType='SUBSET_SELECTION'
    selection=infcSTN
else:
    selection=arcpy.SelectLayerByAttribute_management(infcSTN, "CLEAR_SELECTION", "")



#Determining a further selection options.  Either by SQL query or polygon shapefile
if selectbylocation and not selectionSQL:
    arcpy.AddMessage("Executing: Selectbylocation")
    selection =arcpy.SelectLayerByLocation_management(selection,'within',selectbylocation, '', selectionType)
elif selectbylocation and selectionSQL:
    arcpy.AddMessage("Executing: Selectbylocation and SelectionSQL")
    selection =arcpy.SelectLayerByLocation_management(selection,'within',selectbylocation, '', selectionType)
    selection = arcpy.SelectLayerByAttribute_management(selection,"SUBSET_SELECTION", selectionSQL)
elif selectionSQL:
    arcpy.AddMessage("Executing: selectionSQL")
    selection = arcpy.SelectLayerByAttribute_management(selection,selectionType, selectionSQL)


#Creating a search cursor, and sorting the data based on parameter specified
rows = arcpy.SearchCursor(selection, "", os.path.join(arcpy.GetInstallInfo()["InstallDir"], "Coordinate Systems/Geographic Coordinate Systems/World/WGS 1984.prj"), "", "")


#Booleans for folder checks
firstValue=True
addFolder=False
skipFoler=False


#enumerate through rows in attribute table
rownum=0


numflds = len(fields)
Num_Skipped = 0
Stn_skipped = ""

found = False
for row in rows:
    currSTN  = row.getValue(stationName)


    rownum+=1
    feat = row.getValue(shapeName) #get geometry object
    newFolder=False #assume KMLPhotos/Station ID has been created from previous iteration
    outfiles=[]


    if not os.path.exists(outputPhotoKML + "/KMLPhotos/" + currSTN):
        os.makedirs(outputPhotoKML + "/KMLPhotos/"  + currSTN) #Creating a jpg folder directory
        newFolder=True


    #Finds the first photo
    expr = arcpy.AddFieldDelimiters(infcPHO,stationName) + " = '" + currSTN + "'"
    FirstPhoto = "none"
    arcpy.SelectLayerByAttribute_management(infcPHO, "CLEAR_SELECTION")
    if pselectionSQL:
        frows = arcpy.da.SearchCursor(infcPHO,photoName, where_clause= '(' + expr + ') AND (' + pselectionSQL + ')')
    else:
        frows = arcpy.da.SearchCursor(infcPHO,photoName, where_clause=expr)

    for frow in frows:
        if FirstPhoto == "none":
            FirstPhoto = frow[0]

    if FirstPhoto <> "none":  # in case no photos in filtered subset, don't write placemark

        #Adds the placemark - one for each station
        string = ""
        string+="<Placemark>\n"
        string+="   <name>%s</name>\n"%(currSTN)
        string+='''      <styleUrl>#exampleStyleMap</styleUrl>
              <description>\n'''

        string+='''        <![CDATA[
          <head style="background-color:yellow;">
            <script language="JavaScript">
            function LoadGallery(image,filename) {
            document.getElementById('bigimage').src = filename;
            var element = document.getElementById("picName");
            var str = filename;
            element.innerHTML = str;
            }
            </script>

            <div id="bigpic">
            <center><p><img src='%s' Width = "770" Height = "550"id="bigimage"></div><b id="picName">%s</b><br>''' % ("KMLPhotos/"  + currSTN + "/" + FirstPhoto + ".jpg", "KMLPhotos/"  + currSTN + "/" + FirstPhoto + ".jpg")#This the reference for the large center image in the popup window
        if(fields!=''):
            for field in fields: #enumerate through field list not matching geometry
                string+= "<b> %s:</b> %s" % (field, row.getValue(field))
        if pselectionSQL:
            string+="<p><b> Filter: </b></p> %s" % (pselectionSQL)

        string+='''</center>
            <center><p>Photo Gallery.</p>
            </center>\n
            <center>'''

        #Creating a search cursor, and sorting the data based on parameter specified
        expr = arcpy.AddFieldDelimiters(infcPHO,stationName) + " = '" + currSTN + "'"
        if pselectionSQL:
            prows = arcpy.da.SearchCursor(infcPHO,photoName, where_clause= '(' + expr + ') AND (' + pselectionSQL + ')')
        else:
            prows = arcpy.da.SearchCursor(infcPHO,photoName, where_clause=expr)


        for prow in prows:
            #As long as there is a photo in the photolist, this loop will continue to run

            #Add photo to folder
            currPhoto = prow[0]
            arcpy.AddMessage(currPhoto)
            try:
                shutil.copyfile(inputPhotoDB + "\\" + currPhoto + ".jpg",outputPhotoKML + "\\KMLPhotos\\"  + currSTN + "\\" + currPhoto + ".jpg")
            except IOError:
                print "cannot copy", currPhoto
                arcpy.AddError("Unable to copy file: " + currPhoto + ".jpg.  Please check Your folder")
                sys.exit(1)

            found=True

            #This creates the thumbnail images and has them referencing the 'bigimage' which will switch the thumbnail with the big one in the center.

            string+='''<a onclick="LoadGallery('bigimage','%s')"><img src='%s' height = "50" width = "50"></a>\n''' % ("KMLPhotos/"  + currSTN + "/" + currPhoto + ".jpg", "KMLPhotos/"  + currSTN + "/" + currPhoto + ".jpg")
        string+='''</center>
            </div>
            </head>
        ]]>\n'''

        string+= '''      </description>\n'''
        string+= '''      <Point>
        <coordinates> %s , %s</coordinates>''' % (feat.getPart().X, feat.getPart().Y)
        string+='''       </Point>
        </Placemark>'''
        f.write(string)
    else:
        os.rmdir(outputPhotoKML + "/KMLPhotos/"  + currSTN) #Creating a jpg folder directory
        if Num_Skipped <1:
            Stn_skipped = currSTN
        else:
            Stn_skipped = Stn_skipped + "; " + currSTN
        Num_Skipped += 1

string=""


string+='''</Document>
    </kml>'''
f.write(string)

if Num_Skipped > 0:
    arcpy.AddWarning("-------------")
    arcpy.AddWarning("Warning!")
    arcpy.AddWarning(str(Num_Skipped) + " station(s) with no related photos were not included:")
    arcpy.AddWarning(Stn_skipped)
    arcpy.AddWarning("-------------")

f.close()
